package cn.estsh.i3plus.core.apiservice.controller;

import cn.estsh.i3plus.icloud.form.sdk.IFormDynamicFormCloud;
import cn.estsh.i3plus.pojo.base.bean.BaseResultBean;
import cn.estsh.i3plus.pojo.form.bean.BfDataObject;
import cn.estsh.i3plus.pojo.form.bean.BfDataObjectProperty;
import cn.estsh.i3plus.pojo.form.bean.BfDataSource;
import cn.estsh.i3plus.pojo.model.form.CloudFormModel;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.map.HashedMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @Description :
 * @Reference :
 * @Author : Adair Peng
 * @CreateDate : 2019-06-06 21:09
 * @Modify:
 **/
@RestController
@RequestMapping("/impp/form")
@Api(description="动态报表设置")
public class DemoDynamicFormController {

    private static final Logger LOGGER = LoggerFactory.getLogger(DemoDynamicFormController.class);

    @Autowired
    private IFormDynamicFormCloud formDynamicFormCloud;

    private static CloudFormModel model = null;
    static {
        model = new CloudFormModel();
        model.setSourceCode("form_test");
        model.setDataObjectCode("sys_config");
    }

    /**
     * 测试数据源信息
     * ip:192.168.1.55
     * port:3306
     * database:form_test
     * user:form_test
     * pwd:form_test
     *
     * @return
     */
    @GetMapping("/get-data-source")
    @ApiOperation("获取数据源及动态对象")
    public ResultBean getDataSource(){
        // 查询所有数据源
        BaseResultBean<BfDataSource> baseResultBean = formDynamicFormCloud.findBfDataSource();
        LOGGER.info("全部数据源信息：{}", JSON.toJSONString(baseResultBean));

        // 根据数据源code查询数据源信息
        baseResultBean = formDynamicFormCloud.getBfDataSourceBySourceCode("form_test");
        if(!baseResultBean.isSuccess()){
            return ResultBean.fail(baseResultBean.errorMsg);
        }

        // 查询指定数据源的数据对象（表信息）
        BaseResultBean<BfDataObject> dataObjectList =
                formDynamicFormCloud.findBfDataObjectByDataSourceCode(baseResultBean.getResultObject().getSourceCode());

        return ResultBean.success("查询成功").setResultObject(baseResultBean.getResultObject()).setResultList(dataObjectList.getResultList());
    }

    @PostMapping("/dynamic-entity/insert")
    @ApiOperation(value = "新增数据",notes = "插入单条数据")
    public BaseResultBean insert(){
        try {
            Map<String,Object> insert = new HashedMap();
            insert.put("sid",111);
            insert.put("configType",111);
            insert.put("configValue",111);

            model.setInsert(insert);

            return formDynamicFormCloud.insert(model);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 插入单条数据
     * @return
     */
    @PostMapping("/dynamic-entity/insert-batch")
    @ApiOperation(value = "新增数据",notes = "插入单条数据")
    public BaseResultBean insertBatch(){
        try {
            List<Map<String,Object>> insertList = new ArrayList<>();
            Map<String,Object> insert1 = new HashedMap();
            insert1.put("sid",222);
            insert1.put("configType",222);
            insert1.put("configValue",222);


            Map<String,Object> insert2 = new HashedMap();
            insert2.put("sid",333);
            insert2.put("configType",333);
            insert2.put("configValue",333);

            insertList.add(insert1);
            insertList.add(insert2);
            model.setInsertList(insertList);

            return formDynamicFormCloud.insertBatch(model);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据ID删除指定数据
     */
    @PostMapping("/dynamic-entity/delete")
    @ApiOperation(value = "删除数据",notes = "根据ID删除指定数据")
    public BaseResultBean delete(){
        try {

            Map<String,Object> delete = new HashedMap();
            delete.put("sid",111);
            model.setDelete(delete);

            return formDynamicFormCloud.delete(model);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 批量删除数据
     * @return
     */
    @PostMapping("/dynamic-entity/delete-batch")
    @ApiOperation(value = "批量删除数据",notes = "批量删除数据")
    public BaseResultBean deleteBatch(){
        try {
            List<Map<String,Object>> deleteList = new ArrayList<>();
            Map<String,Object> delete1 = new HashedMap();
            delete1.put("sid",222);
            delete1.put("configType",222);
            delete1.put("configValue",222);


            Map<String,Object> delete2 = new HashedMap();
            delete2.put("sid",333);
            delete2.put("configType",333);
            delete2.put("configValue",333);

            deleteList.add(delete1);
            deleteList.add(delete2);
            model.setDeleteList(deleteList);

            return formDynamicFormCloud.deleteBatch(model);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


    /**
     * 修改单条数据
     * @return
     */
    @PostMapping("/dynamic-entity/update")
    @ApiOperation(value = "修改数据",notes = "修改单条数据")
    public BaseResultBean update(){
        try {
            Map<String,Object> update = new HashedMap();
            update.put("sid",111);
            update.put("configType",000);
            update.put("configValue",000);

            model.setUpdate(update);

            return formDynamicFormCloud.update(model);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 修改多条数据
     * @return
     */
    @PostMapping("/dynamic-entity/update-batch")
    @ApiOperation(value = "批量修改数据",notes = "批量修改数据")
    public BaseResultBean updateBatch(){
        try {
            List<Map<String,Object>> updateList = new ArrayList<>();
            Map<String,Object> update1 = new HashedMap();
            update1.put("sid",222);
            update1.put("configType",000);
            update1.put("configValue",000);


            Map<String,Object> update2 = new HashedMap();
            update2.put("sid",333);
            update2.put("configType",999);
            update2.put("configValue",999);

            updateList.add(update1);
            updateList.add(update2);
            model.setUpdateList(updateList);

            return formDynamicFormCloud.updateBatch(model);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据ID查询单条数据
     * @return 处理结果
     */
    @PostMapping("/dynamic-entity/get-primary")
    @ApiOperation(value = "查询数据",notes = "根据ID查询单条数据")
    public BaseResultBean getDynamicEntity(){
        try {
            BfDataObjectProperty property = new BfDataObjectProperty();
            property.setPropertyCode("sid");
            property.setPropertyFormValue("111");

            model.setSelectList(new ArrayList<BfDataObjectProperty>(){{
                add(property);
            }});

            return formDynamicFormCloud.getDynamicEntityByPrimary(model);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据条件查询所有
     * @return 处理结果
     */
    @PostMapping("/dynamic-entity/find")
    @ApiOperation(value = "查询数据",notes = "根据条件查询所有")
    public BaseResultBean findDynamicEntity(){
        try {
            BfDataObjectProperty property = new BfDataObjectProperty();
            property.setPropertyCode("configType");
            property.setPropertyFormValue("22");

            BfDataObjectProperty property2 = new BfDataObjectProperty();
            property2.setPropertyCode("configValue");
            property2.setPropertyFormValue("22");

            model.setSelectList(new ArrayList<BfDataObjectProperty>(){{
                add(property);
                add(property2);
            }});

            return formDynamicFormCloud.findDynamicEntity(model);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {

            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


}
