package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ICoreMemTreeService;
import cn.estsh.i3plus.core.api.iservice.busi.IPersonnelService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysPositionService;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.model.platform.CommonTreeModel;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;
import java.util.List;

/**
 * @Description : 内存树
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 2019-07-04 下午12:52
 * @Modify:
 **/
@RestController
@Api(description = "内存树服务")
@RequestMapping(PlatformConstWords.BASE_URL + "/mem-tree")
public class SysMemTreeController extends CoreBaseController {

    @Autowired
    private ISysPositionService positionService;

    @Autowired
    private ICoreMemTreeService memTreeService;

    @Autowired
    private IPersonnelService personnelService;

    /**
     * 根据父节点ID 获取树列表
     * @return 处理结果
     */
    @GetMapping(value="/position/root")
    @ApiOperation(value="查询岗位树",notes="通过部门主键获取岗位树，-1为根节点")
    public ResultBean listSysPositionTree() {
        try {
            List list = positionService.findSysPositionAll();
            List treeList = memTreeService.packTreeSysPosition(list, CommonEnumUtil.PARENT.DEFAULT.getValue());

            return ResultBean.success("查询部门树成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(treeList);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据用户ID 获取用户组织树
     * @return
     */
    @GetMapping(value="/position/user")
    @ApiOperation(value="查询岗位树",notes="通过部门主键获取岗位树，-1为根节点")
    public ResultBean listSysPositionParentId() {
        try {
            List<SysPosition> result = null;

            if (AuthUtil.getUserType().equals(CommonEnumUtil.USER_TYPE.ADMIN)) {
                result = positionService.findSysPositionAll();
            }else{
                result = positionService.findSysPositionByOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            }

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(memTreeService.packTreeSysPosition(result, CommonEnumUtil.PARENT.DEFAULT.getValue()));
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 分装树
     * @param ids
     * @return
     */
    @GetMapping(value="/position/ids")
    @ApiOperation(value="查询岗位树",notes="通过部门主键获取岗位树，-1为根节点")
    public ResultBean listSysPositionParentId(Long[] ids) {
        try {
            List list = positionService.findSysPositionAll();
            List listTree = memTreeService.packTreeSysPosition(list, CommonEnumUtil.PARENT.DEFAULT.getValue());

            if (AuthUtil.getUserType().getValue() == CommonEnumUtil.USER_TYPE.USER.getValue()) {
                List dataListTree = memTreeService.packTreeSysPositionByIds(listTree, Arrays.asList(ids));
                return ResultBean.success("查询部门树成功")
                        .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                        .setResultList(dataListTree);
            } else {
                return ResultBean.success("查询部门树成功")
                        .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                        .setResultList(listTree);
            }

        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 获取指定ID 节点的树 , 自动补充跟节点
     * @param id
     * @return
     */
    @GetMapping(value="/position/id")
    @ApiOperation(value="查询岗位树",notes="通过部门主键获取岗位树，-1为根节点")
    public ResultBean listSysPositionById(Long id) {
        try {
            List list = positionService.findSysPositionAll();
            SysPosition position = memTreeService.packTreeSysPositionById(list,id);

            return ResultBean.success("查询部门树成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultObject(position);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 获取字节点所有的ID
     * @param id
     * @return
     */
    @GetMapping(value="/position/child")
    @ApiOperation(value="查询岗位树",notes="通过部门主键获取岗位树，-1为根节点")
    public ResultBean listSysPositionChild(Long id) {
        try {
            List list = positionService.findSysPositionAll();
            SysPosition position = memTreeService.packTreeSysPositionById(list,id);

            return ResultBean.success("查询部门树成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(memTreeService.findChildSysPosition(position));
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据用户ID 获取用户组织树
     * @return
     */
    @GetMapping(value="/organize/user")
    @ApiOperation(value="查询组织树",notes="通过部门主键获取组织树，-1为根节点")
    public ResultBean findSysOrganizeByUser() {
        try {
            List<SysOrganize> list = personnelService.findSysOrganizeList();
            List listTree = memTreeService.packTreeSysOrganize(list, CommonEnumUtil.PARENT.DEFAULT.getValue());

            if (AuthUtil.getUserType().getValue() == CommonEnumUtil.USER_TYPE.USER.getValue()) {
                List<SysRefUserDepartment> refList = personnelService.findSysRefUserDepartmentByUserId(getSessionUser().getUser().getId());
                List userTreeList = memTreeService.packTreeSysOrganizeBySysRefUserOrganize(listTree, refList);

                return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(userTreeList);
            } else {
                return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(listTree);
            }

        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据用户ID 获取用户部门树
     * @return
     */
    @GetMapping(value="/organize-department/user")
    @ApiOperation(value="查询部门树",notes="通过部门主键获取部门树，-1为根节点")
    public ResultBean findTreeModelUser() {
        try {
            List<CommonTreeModel> result = null;

            if(AuthUtil.getUserType().equals(CommonEnumUtil.USER_TYPE.ADMIN)){
                result = memTreeService.packTreeTreeModelRoot();
            }else {
                result = memTreeService.packTreeTreeModelByUserId(getSessionUser().getUser().getId());
            }

            return ResultBean.success("操作成功").setResultList(result).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value="/menu/user")
    @ApiOperation(value="查询岗位树",notes="通过部门主键获取岗位树，-1为根节点")
    public ResultBean findTreeMenuUser() {
        try {
            List<SysMenu> result = null;
            if(AuthUtil.getUserType().equals(CommonEnumUtil.USER_TYPE.ADMIN)){
                result = memTreeService.packTreeSysMenu();
            }else{
                result = memTreeService.packTreeSysMenuByUserId(getSessionUser().getUser().getId());
            }
            return ResultBean.success("操作成功").setResultList(result).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
}
