package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.IPersonnelService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysMenuService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.ExcelTool;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysMenu;
import cn.estsh.i3plus.pojo.platform.bean.SysRefRoleMenu;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.persistence.EntityManager;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description : 系统功能对外接口
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-22 15:18:14.982
 * @Modify :
 **/
@RestController
@Api(description = "系统功能服务")
@RequestMapping(PlatformConstWords.BASE_URL + "/sys-menu")
public class SysMenuController extends CoreBaseController {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysMenuController.class);

    @Autowired
    private ISysMenuService sysMenuService;

    @Autowired
    private EntityManager entityManager;

    @Resource(name="redisRes")
    private ImppRedis redisRes;

    @Autowired
    private IPersonnelService personnelService;

    /**
     * 新增系统功能
     * @param menu 系统功能
     * @return 处理结果
     */
    @PostMapping(value = "/insert")
    @ApiOperation(value = "新增系统功能", notes = "系统功能")
    public ResultBean insertSysMenu(SysMenu menu) {
        try {
            //条件验证
            ValidatorBean.beginValid(menu)
                    .notNull("name", menu.getName())
                    .notNull("menuCode", menu.getMenuCode())
                    .notNull("menuType", menu.getMenuType());

            //新增初始化
            ConvertBean.modelInitialize(menu, getSessionUser());
            menu.setNameZhShortening(StringTool.getAllFirstLetter(menu.getName()));
            if (menu.getParentId() == null || menu.getParentId().intValue() == 0) {
                menu.setParentId(CommonEnumUtil.PARENT.DEFAULT.getValue());
            }
            if (menu.getMenuSort() == null) {
                menu.setMenuSort(0);
            }

            menu.setMenuStatus(CommonEnumUtil.DATA_STATUS.ENABLE.getValue());

            menu = sysMenuService.insertSysMenu(menu);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(menu);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep).build();
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 修改系统功能
     * @param menu 系统功能
     * @return 处理结果
     */
    @PutMapping(value = "/update")
    @ApiOperation(value = "修改系统功能", notes = "系统功能")
    public ResultBean updateSysMenu(SysMenu menu) {
        try {
            // 数据校验
            ValidatorBean.beginValid(menu)
                    .notNull("id", menu.getId())
                    .notNull("menuType", menu.getMenuType())
                    .notNull("menuCode", menu.getMenuCode())
                    .notNull("name", menu.getName());

            ConvertBean.modelUpdate(menu, getSessionUser());
            menu.setNameZhShortening(StringTool.getAllFirstLetter(menu.getName()));
            if (menu.getParentId() == null || menu.getParentId().intValue() == 0) {
                menu.setParentId(CommonEnumUtil.PARENT.DEFAULT.getValue());
            }
            if (menu.getMenuSort() == null) {
                menu.setMenuSort(0);
            }

            sysMenuService.updateSysMenu(menu);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep).build();
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 删除系统功能
     * @param id 系统功能id
     * @return 处理结果
     */
    @DeleteMapping(value = "/delete/{id}")
    @ApiOperation(value = "删除系统功能", notes = "删除系统功能")
    public ResultBean deleteSysMenu(@PathVariable("id") String id) {
        try {
            // 数据校验
            ValidatorBean.checkIsNumber(id, "系统功能id 不能为空");

            sysMenuService.deleteSysMenuById(Long.parseLong(id));
            return ResultBean.success().setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询所有系统功能
     * @return 处理结果
     */
    @GetMapping(value = "/list")
    @ApiOperation(value = "查询所有系统功能", notes = "查询所有系统功能")
    public ResultBean findAll() {
        try {
            List<SysMenu> list = sysMenuService.findSysMenuAll();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(list);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询权限
     * @param menu 查询条件
     * @param pager 分页数据
     * @return 处理结果
     */
    @GetMapping(value = "/query")
    @ApiOperation(value = "查询权限", notes = "组合查询权限，外带分页功能")
    public ResultBean querySysMenu(SysMenu menu, Pager pager) {
        try {
            ListPager<SysMenu> result = null;
            if(AuthUtil.getUserType().equals(CommonEnumUtil.USER_TYPE.ADMIN)){
                result = sysMenuService.querySysMenu(menu, pager);
            }else {
                List<SysRefRoleMenu> refList = personnelService.findSysRefRoleMenuByUserId(getSessionUser().getUser().getId());
                if(refList != null && refList.size() > 0){
                    List<Long> idList = new ArrayList<>(refList.size());
                    refList.forEach(ref -> idList.add(ref.getMenuId()));
                    result = sysMenuService.querySysMenu(menu,idList,pager);
                }
            }

            return ResultBean.success("操作成功").setListPager(result).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询系统功能
     * @param id 系统功能id
     * @return 处理结果
     */
    @GetMapping(value = "/get/{id}")
    @ApiOperation(value = "查询系统功能", notes = "查询根据ID查询系统功能")
    public ResultBean getById(@PathVariable("id") String id) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "功能id 不能为空");

            SysMenu menu = sysMenuService.getSysMenuById(Long.parseLong(id));

            if (menu != null) {
                return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(menu);
            } else {
                return ResultBean.fail("数据不存在").setCode(ResourceEnumUtil.MESSAGE.EMPTY.getCode());
            }
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }

    }

    /**
     * 修改系统功能状态
     * @param id 系统功能id
     * @param status 状态
     * @return 处理结果
     */
    @PutMapping(value = "/status")
    @ApiOperation(value = "修改系统功能状态", notes = "根据ID修改系统功能状态")
    public ResultBean updateStatus(String id, int status) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "功能id 不能为空");
            ValidatorBean.checkNotZero(status, "功能状态不能为空");

            sysMenuService.updateSysMenuStatus(Long.parseLong(id), status, getSessionUser());
            return new ResultBean(true);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 批量修改系统功能状态
     * @param ids 系统功能id
     * @param status 系统状态
     * @return 处理结果
     */
    @PutMapping(value = "/batch-status")
    @ApiOperation(value = "批量修改系统功能状态", notes = "根据ID修改系统功能状态")
    public ResultBean updateBatchStatus(String[] ids, int status) {
        try {
            // 数据校验
            ids = ConvertBean.modelSafeArrayNumber(ids, true);
            ValidatorBean.checkNotNull(ids, "功能id 不能为空");
            ValidatorBean.checkNotZero(status, "功能状态不能为空");

            sysMenuService.updateSysMenuStatusByIds(StringTool.getArrayLong(ids), status, getSessionUser());

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据id批量删除
     * @param ids 系统功能id数组
     * @return 处理结果
     */
    @DeleteMapping(value = "/batch-delete")
    @ApiOperation(value = "根据id批量删除", notes = "根据id批量删除")
    public ResultBean deleteSysMenuByIds(String[] ids) {
        try {
            ids = ConvertBean.modelSafeArrayNumber(ids, true);
            if (ids.length == 0) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要删除的资源。")
                        .build();
            }
            sysMenuService.deleteSysMenuByIds(StringTool.getArrayLong(ids));
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }



    /**
     * 文件上传
     * @param multipart 文件
     * @return 处理结果
     */
    @PostMapping("/upload")
    @ApiOperation(value = "文件上传", notes = "文件上传")
    public ResultBean singleFileUpload(@RequestParam("file") MultipartFile multipart) {
        try {
            startMultiService();
            ExcelTool excelTool = new ExcelTool(entityManager, redisRes);

            if(multipart.isEmpty()){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要上传的文件。")
                        .build();
            }
            List<SysMenu> list = excelTool.importData("permission.xls",multipart.getInputStream(), SysMenu.class);

            Map<String,SysMenu> map = new HashMap<>();
            list.forEach(menu -> {
                ValidatorBean.checkNotNull(menu.getMenuCode(), "功能代码不能为空");
                ValidatorBean.checkNotNull(menu.getParentId(), "父级功能ID不能为空");

                menu.setId(Long.parseLong(menu.getMenuCode()));
                menu.setMenuSort(10000);
                if (!map.containsKey(menu.getMenuCode())) {
                    map.put(menu.getMenuCode(),menu);
                } else {
                    throw ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                            .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                            .setErrorDetail("功能代码【" + menu.getMenuCode() + "】重复")
                            .build();
                }

            });

            list.forEach(menu -> {
                menu.setIsDeleted(CommonEnumUtil.IS_DEAL.NO.getValue());
                menu.setIsValid(CommonEnumUtil.IS_VAILD.VAILD.getValue());
                if(menu.getParentId().longValue() != -1){
                    SysMenu mu = map.get(menu.getParentId().toString());
                    ValidatorBean.checkNotNull(mu, "【"+menu.getParentId()+"】不存在的父节点信息");
                    menu.setNameZhShortening(StringTool.getAllFirstLetter(mu.getName()));
                    menu.setParentNameRdd(mu.getName());
                }
            });

            return ResultBean.success("上传成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                                .setResultList(sysMenuService.refreshSysMenu(list));
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 文件下载
     *
     * @param response response
     * @return 处理结果
     */
    @GetMapping("/download/excel")
    @ApiOperation(value = "权限文件下载", notes = "权限文件下载")
    public ResultBean singleDownload(HttpServletResponse response) {
        try {
            startMultiService();
            ExcelTool excelTool = new ExcelTool(entityManager, redisRes);
            String fileName = "menu_" + System.currentTimeMillis() + ".xls";
            List<SysMenu> list = sysMenuService.findSysMenuAll();

            File file = new File(fileName);
            file.createNewFile();
            File excle = excelTool.exportData(file, list, SysMenu.class, new String[]{
                    "menuCode", "name","parentId","menuType","menuStatus","parentNameRdd","menuUrl","menuClassPath","menuGrade","menuSort"
                    ,"menuCss","menuIcon","menuDescription"
            });

            InputStream targetStream = new DataInputStream(new FileInputStream(excle));
            response.setContentType("application/force-download"); // 设置强制下载不打开
            response.addHeader("Content-Disposition", "attachment;fileName=" + fileName); // 设置文件名
            response.addHeader("Content-type", "application/octet-stream"); // 设置文件名

            BufferedInputStream bis = null;
            try {
                bis = new BufferedInputStream(targetStream);
                OutputStream os = response.getOutputStream();
                byte[] buffer = new byte[1024];
                int i = bis.read(buffer);
                while (i != -1) {
                    os.write(buffer, 0, i);
                    i = bis.read(buffer);
                }
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                if (bis != null) {
                    try {
                        bis.close();
                        targetStream.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
            file.delete(); // 删除临时文件
            return ResultBean.success("下载").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
