package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.base.ISystemInitService;
import cn.estsh.i3plus.core.api.iservice.busi.ICoreMemTreeService;
import cn.estsh.i3plus.core.api.iservice.busi.IPersonnelService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysOrganizeService;
import cn.estsh.i3plus.icloud.wms.sdk.IWmsActionGroupCloud;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.BaseResultBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysConfig;
import cn.estsh.i3plus.pojo.platform.bean.SysOrganize;
import cn.estsh.i3plus.pojo.platform.bean.SysRefUserDepartment;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description : 组织对外接口
 * @Reference   :
 * @Author      : wei.peng
 * @Date        : 2018-10-22 16:58:42.972
 * @Modify      :
 **/
@RestController
@Api(description = "组织")
@RequestMapping(PlatformConstWords.BASE_URL +"/organize")
public class SysOrganizeController extends CoreBaseController{

    public static final Logger LOGGER = LoggerFactory.getLogger(SysOrganizeController.class);

    @Autowired
    private ISysOrganizeService organizeService;

    @Autowired
    private IPersonnelService personnelService;

    @Autowired
    private ICoreMemTreeService coreMemTreeService;

    @Autowired
    private ISystemInitService systemInitService;

    @Autowired
    private IWmsActionGroupCloud wmsActionGroupCloud;

    /**
     * 新增组织
     * @param organize 组织
     * @return 处理结果
     */
    @PostMapping(value = "/insert")
    @ApiOperation(value = "新增组织", notes = "组织")
    public ResultBean insertSysOrganize(SysOrganize organize) {
        try {
            //条件验证
            ValidatorBean.beginValid(organize)
                    .notNull("name", organize.getName())
                    .notNull("organizeCode", organize.getOrganizeCode())
                    .notNull("organizeType", organize.getOrganizeType());

            //新增初始化
            ConvertBean.modelInitialize(organize, AuthUtil.getSessionUser());
            if(organize.getParentId() == null || organize.getParentId() <= 0){
                organize.setParentId(CommonEnumUtil.PARENT.DEFAULT.getValue());
                organize.setParentNameRdd("顶级组织");
            }

            organize = organizeService.insertSysOrganize(organize);
            String resutl = "组织已创建完成";

            String copyMessage = copyWmsProcess(organize);
            resutl = StringUtils.isNotBlank(copyMessage) ? resutl +","+ copyMessage : resutl;
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setMsg(resutl)
                    .setResultObject(organize);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 修改组织
     * @param organize 组织
     * @return 处理结果
     */
    @PutMapping(value = "/update")
    @ApiOperation(value = "修改组织", notes = "组织")
    public ResultBean updateSysOrganize(SysOrganize organize) {
        try {
            // 数据校验
            ValidatorBean.beginValid(organize)
                    .notNull("id", organize.getId())
                    .notNull("name", organize.getName())
                    .notNull("organizeCode", organize.getOrganizeCode())
                    .notNull("organizeType", organize.getOrganizeType());

            ConvertBean.modelUpdate(organize, AuthUtil.getSessionUser());
            if(organize.getParentId() == null || organize.getParentId() <= 0){
                organize.setParentId(CommonEnumUtil.PARENT.DEFAULT.getValue());
                organize.setParentNameRdd("顶级组织");
            }

            organizeService.updateSysOrganize(organize);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 删除组织
     * @param idStr 组织id
     * @return 处理结果
     */
    @DeleteMapping(value = "/delete/{id}")
    @ApiOperation(value = "删除组织", notes = "删除组织")
    public ResultBean deleteSysOrganize(@PathVariable("id") String idStr) {
        try {
            // 数据校验
            ValidatorBean.checkIsNumber(idStr.toString(), "您要删除的组织不存在");

            organizeService.deleteSysOrganizeById(Long.parseLong(idStr));
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询所有组织
     * @return 处理结果
     */
    @GetMapping(value = "/list")
    @ApiOperation(value = "查询所有组织", notes = "查询所有组织")
    public ResultBean findSysOrganizeAll() {
        try {
            List<SysOrganize> result;
            if(AuthUtil.getUserType().equals(CommonEnumUtil.USER_TYPE.ADMIN)){
                result = organizeService.findSysOrganizeList();
            }else {
                List<SysRefUserDepartment> refList = personnelService.findSysRefUserDepartmentByUserId(getSessionUser().getUser().getId());
                result = organizeService.findOrganizeIdByRefUsers(refList);
            }

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(result);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询组织
     * @param organize 查询条件
     * @param pager 分页信息
     * @return 处理结果
     */
    @GetMapping(value="/query")
    @ApiOperation(value="查询组织",notes="组合查询组织外加分页")
    public ResultBean queryPosition(SysOrganize organize, Pager pager){
        try{
            List<Long> idList = new ArrayList<>();

            if(AuthUtil.getUserType().equals(CommonEnumUtil.USER_TYPE.ADMIN)){
                if(organize.getParentId() != null){
                    idList = coreMemTreeService.findSysOrganizeChildIdList(organize.getParentId());
                }
            }else{
                List<SysRefUserDepartment> refList = personnelService.findSysRefUserDepartmentByUserId(getSessionUser().getUser().getId());
                if(organize.getParentId() == null || organize.getParentId() <= 0){
                    if(refList != null && refList.size() > 0){
                        for (SysRefUserDepartment ref : refList) {
                            idList.add(ref.getOrganizeId());
                        }
                    }
                }else{
                    List<SysOrganize> rootTreeList = personnelService.findSysOrganizeList();
                    rootTreeList = coreMemTreeService.packTreeSysOrganize(rootTreeList,CommonEnumUtil.PARENT.DEFAULT.getValue());
                    rootTreeList = coreMemTreeService.packTreeSysOrganizeBySysRefUserOrganize(rootTreeList,refList);
                    SysOrganize sysOrganizeTree = coreMemTreeService.getTreeSysOrganizeById(rootTreeList, organize.getParentId());
                    idList = coreMemTreeService.findChildSysOrganize(sysOrganizeTree);
                }
            }

            ListPager<SysOrganize> result = organizeService.querySysOrganize(organize, idList, pager);
            return ResultBean.success("操作成功").setListPager(result).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询组织
     * @param id 组织id
     * @return 处理结果
     */
    @GetMapping(value = "/get/{id}")
    @ApiOperation(value = "查询组织", notes = "查询根据ID查询组织")
    public ResultBean getBySysOrganizeId(@PathVariable("id") String id) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "角色id 不能为空");

            SysOrganize organize = organizeService.getSysOrganizeById(Long.parseLong(id));

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(organize);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 修改组织状态
     * @param id 组织id
     * @param status 状态
     * @return 处理结果
     */
    @PutMapping(value = "/status")
    @ApiOperation(value = "修改组织状态", notes = "根据ID修改组织状态")
    public ResultBean updateStatus(String id, int status) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "组织id 不能为空");
            ValidatorBean.checkNotZero(status, "组织状态不能为空");

            organizeService.updateSysOrganizeStatus(Long.parseLong(id), status, AuthUtil.getSessionUser());
            return new ResultBean(true);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据id批量删除
     * @param ids 组织ids数组
     * @return 处理结果
     */
    @DeleteMapping(value = "/batch-delete")
    @ApiOperation(value = "根据id批量删除", notes = "根据id批量删除")
    public ResultBean deleteSysOrganizeByIds(String[] ids){
        try{
            ids = ConvertBean.modelSafeArrayNumber(ids,true);
            if(ids.length == 0){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要删除的资源。")
                        .build();
            }

            organizeService.deleteSysOrganizeByIds(StringTool.getArrayLong(ids));
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    private String copyWmsProcess(SysOrganize organize){
        String result = null;
        // 检查是否复制 WMS 流程
        String redisKey = CommonConstWords.REDIS_PREFIX_CACHE_CONFIG + "_" + CommonConstWords.CONFIG_ORGANIZE_INSERT_INIT_WMS_COPY_PROCESS;
        SysConfig config = (SysConfig) systemInitService.getDataFromCache(redisKey, SysConfig.class);

        LOGGER.info("Copy Wms Process Config:{}",config);
        if(config != null){
            if(config.getConfigValue().equals(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue())){

                // 获取目标组织代码
                redisKey = CommonConstWords.REDIS_PREFIX_CACHE_CONFIG + "_" + CommonConstWords.CONFIG_ORGANIZE_INSERT_INIT_WMS_COPY_PROCESS_CODE;
                config = (SysConfig) systemInitService.getDataFromCache(redisKey, SysConfig.class);

                LOGGER.info("Copy Wms Process Organize Code Config:{}",config);
                if(config != null){
                    // 获取组织信息
                    SysOrganize tmpOrganize = organizeService.getSysOrganizeByCode(config.getConfigValue());

                    // 检查组织信息是否完整
                    if(tmpOrganize != null){
                        // 复制WMS 业务流程
                        LOGGER.info("Copy Wms Process Template Code :{} Target Code:{}",tmpOrganize.getOrganizeCode(),organize.getOrganizeCode());
                        BaseResultBean resultBean = wmsActionGroupCloud.copyWmsActionGroupByOrganizeCode(tmpOrganize.getOrganizeCode(), organize.getOrganizeCode());
                        LOGGER.info("Copy Wms Process Template Code :{} Target Code:{} Result Bean:{}",tmpOrganize.getOrganizeCode(),organize.getOrganizeCode(),resultBean);

                        if(resultBean != null){
                            if(ResourceEnumUtil.MESSAGE.SUCCESS.getCode().equals(resultBean.getCode())){
                                result = "流程复制成功";
                            }else{
                                result = resultBean.getErrorMsg();
                            }
                        }else {
                            result = "复制流程失败,服务调用返回信息为空";
                        }
                    }
                }
            }
        }
        return result;
    }

}
