package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysTaskCycleService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysTaskPlanService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysTaskService;
import cn.estsh.i3plus.icloud.andon.sdk.IAndonCommonCloud;
import cn.estsh.i3plus.icloud.andon.sdk.IAndonScheduleJobCloud;
import cn.estsh.i3plus.icloud.mes.pcn.sdk.IMesPcnScheduleJobCloud;
import cn.estsh.i3plus.icloud.mes.sdk.IMesScheduleJobCloud;
import cn.estsh.i3plus.icloud.wms.sdk.IWmsScheduleJobCloud;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.BaseResultBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysTask;
import cn.estsh.i3plus.pojo.platform.bean.SysTaskCycle;
import cn.estsh.i3plus.pojo.platform.bean.SysTaskPlan;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.quartz.IQuartzJobService;
import cn.estsh.impp.framework.boot.quartz.ScheduleTool;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import com.netflix.discovery.converters.Auto;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Description : 任务计划管理服务
 * @Reference :
 * @Author : yunhao
 * @Date : 2018-12-27 10:45
 * @Modify :
 **/
@RestController
@Api(description = "任务计划管理服务")
@RequestMapping(PlatformConstWords.BASE_URL + "/task-plan")
public class SysTaskPlanController extends CoreBaseController {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysTaskPlanController.class);

    @Autowired
    private ISysTaskPlanService sysTaskPlanService;

    @Autowired
    private ISysTaskCycleService sysTaskCycleService;

    @Autowired
    private ISysTaskService sysTaskService;

    /************ 微服定时任务 *********/
    @Autowired
    private ScheduleTool scheduleTool;

    @Autowired
    private IWmsScheduleJobCloud wmsScheduleJobCloud;

    @Autowired
    private IMesScheduleJobCloud mesScheduleJobCloud;

    @Autowired
    private IMesPcnScheduleJobCloud mesPcnScheduleJobCloud;

    @Autowired
    private IAndonScheduleJobCloud andonScheduleJobCloud;

    /**
     * 根据来源切换定时任务服务
     *
     * @param source CommonEnumUtil.SOFT_TYPE
     * @return 定时任务服务
     */
    private IQuartzJobService getJobService(int source) {
        IQuartzJobService quartzJobService = scheduleTool;

        //平台定时任务
        if (source == CommonEnumUtil.SOFT_TYPE.CORE.getValue()) {
            quartzJobService = scheduleTool;
            // WMS
        } else if (source == CommonEnumUtil.SOFT_TYPE.WMS.getValue()) {
            quartzJobService = wmsScheduleJobCloud;
            // MES
        } else if (source == CommonEnumUtil.SOFT_TYPE.MES.getValue()) {
            quartzJobService = mesScheduleJobCloud;
            // MES_PCN
        } else if (source == CommonEnumUtil.SOFT_TYPE.MES_PCN.getValue()) {
            quartzJobService = mesPcnScheduleJobCloud;
            // ANDON
        } else if(source == CommonEnumUtil.SOFT_TYPE.ANDON.getValue()){
            quartzJobService = andonScheduleJobCloud;
        }
        return quartzJobService;
    }

    /**
     * 新增任务计划
     *
     * @param sysTaskPlan 任务计划
     * @param isExecute   是否立即执行
     * @return 处理结果
     */
    @PostMapping(value = "/insert")
    @ApiOperation(value = "新增任务计划", notes = "任务计划")
    public ResultBean insertSysTaskTime(SysTaskPlan sysTaskPlan, Integer isExecute) {
        try {
            startMultiService();

            // 数据校验
            ValidatorBean.beginValid(sysTaskPlan)
                    .notNull("name", sysTaskPlan.getName())
                    .notNull("groupName", sysTaskPlan.getGroupName())
                    .notNull("taskCycleId", sysTaskPlan.getTaskCycleId())
                    .notNull("taskId", sysTaskPlan.getTaskId())
                    .notNull("taskMethodType", sysTaskPlan.getTaskMethodType());

            SysTaskCycle sysTaskCycle = sysTaskCycleService.getSysTaskCycleById(sysTaskPlan.getTaskCycleId());
            ValidatorBean.checkNotNull(sysTaskCycle, "任务周期不存在");

            SysTask sysTask = sysTaskService.getSysTaskById(sysTaskPlan.getTaskId());
            ValidatorBean.checkNotNull(sysTask, "任务不存在");

            // 设置冗余信息
            sysTaskPlan.setTaskNameRdd(sysTask.getName());
            sysTaskPlan.setTaskSoftTypeRdd(sysTask.getTaskSoftType());
            sysTaskPlan.setTaskCycleNameRdd(sysTaskCycle.getName());
            sysTaskPlan.setTaskCycleExpsRdd(sysTaskCycle.getTaskCycleExps());
            sysTaskPlan.setTaskCycleStartDateTimeRdd(sysTaskCycle.getTaskCycleStartDatetime());
            sysTaskPlan.setTaskCycleEndDateTimeRdd(sysTaskCycle.getTaskCycleEndDatetime());
            sysTaskPlan.setTaskPlanExecNum(0);
            sysTaskPlan.setTaskPlanStatus(CommonEnumUtil.DATA_STATUS.ENABLE.getValue());

            // 唯一检查
            LOGGER.info("定时任务唯一校验 Name:{0}，GroupName:{1}", sysTaskPlan.getName(), sysTaskPlan.getGroupName());
            BaseResultBean baseResultBean = getJobService(sysTaskPlan.getTaskSoftTypeRdd())
                    .checkJobExists(sysTaskPlan.getName(), sysTaskPlan.getGroupName());
            if (baseResultBean.isSuccess() && baseResultBean.getResultObject() != null && (boolean) baseResultBean.getResultObject()) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                        .setErrorDetail("定时任务已存在" + baseResultBean.getErrorMsg())
                        .build();
            }

            // 添加至对应模块调度器中
            baseResultBean = getJobService(sysTaskPlan.getTaskSoftTypeRdd()).addTask(sysTask.getTaskPackage(),
                    sysTask.getTaskClass(), sysTaskPlan.getName(), sysTaskPlan.getGroupName(), sysTaskCycle.getTaskCycleExps(),
                    sysTaskPlan.getTaskPlanParam());
            if (!baseResultBean.isSuccess()) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                        .setErrorDetail("定时任务添加失败-" + baseResultBean.getErrorMsg())
                        .build();
            }

            sysTaskPlan = sysTaskPlanService.insertSysTaskPlan(sysTaskPlan);

            // 立即执行
            if (isExecute != null && isExecute == CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()) {
                baseResultBean = getJobService(sysTaskPlan.getTaskSoftTypeRdd())
                        .executeJob(sysTaskPlan.getName(), sysTaskPlan.getGroupName());
                if (!baseResultBean.isSuccess()) {
                    throw ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                            .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                            .setErrorDetail("定时任务执行失败-" + baseResultBean.getErrorMsg())
                            .build();
                }
            }
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(sysTaskPlan);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 修改任务计划
     *
     * @param sysTaskPlan 任务计划对象
     * @param isExecute   是否立即执行
     * @return 处理结果
     */
    @PutMapping(value = "/update")
    @ApiOperation(value = "修改任务计划", notes = "任务计划")
    public ResultBean updateSysTaskTime(SysTaskPlan sysTaskPlan, Integer isExecute) {
        try {
            startMultiService();

            ResultBean resultBean = ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultObject(sysTaskPlan);
            // 数据校验
            ValidatorBean.beginValid(sysTaskPlan)
                    .notNull("name", sysTaskPlan.getName())
                    .notNull("groupName", sysTaskPlan.getGroupName())
                    .notNull("taskCycleId", sysTaskPlan.getTaskCycleId())
                    .notNull("taskId", sysTaskPlan.getTaskId())
                    .notNull("taskMethodType", sysTaskPlan.getTaskMethodType());

            SysTaskCycle sysTaskCycle = sysTaskCycleService.getSysTaskCycleById(sysTaskPlan.getTaskCycleId());
            ValidatorBean.checkNotNull(sysTaskCycle, "任务周期不存在");

            SysTask sysTask = sysTaskService.getSysTaskById(sysTaskPlan.getTaskId());
            ValidatorBean.checkNotNull(sysTask, "任务不存在");

            sysTaskPlan.setTaskNameRdd(sysTask.getName());
            sysTaskPlan.setTaskSoftTypeRdd(sysTask.getTaskSoftType());
            sysTaskPlan.setTaskCycleNameRdd(sysTaskCycle.getName());
            sysTaskPlan.setTaskCycleExpsRdd(sysTaskCycle.getTaskCycleExps());
            sysTaskPlan.setTaskCycleStartDateTimeRdd(sysTaskCycle.getTaskCycleStartDatetime());
            sysTaskPlan.setTaskCycleEndDateTimeRdd(sysTaskCycle.getTaskCycleEndDatetime());

            sysTaskPlanService.updateSysTaskPlan(sysTaskPlan);

            // 调用对应微服务的检查接口
            LOGGER.info("微服定时任务数据校验：name:{}, groupName:{}", sysTaskPlan.getName(), sysTaskPlan.getGroupName());
            BaseResultBean baseResultBean = getJobService(sysTaskPlan.getTaskSoftTypeRdd())
                    .checkJobExists(sysTaskPlan.getName(), sysTaskPlan.getGroupName());
            if (baseResultBean.isSuccess() && baseResultBean.getResultObject() != null && !(boolean) baseResultBean.getResultObject()) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                        .setErrorDetail("定时任务不存在")
                        .build();
            } else if (!baseResultBean.isSuccess()) {
                resultBean = ResultBean.fail(baseResultBean.getErrorMsg());
            }

            // 修改对应微服务的定时任务
            LOGGER.info("微服修改定时任务数据：taskPlan", sysTaskPlan.toString());
            baseResultBean = getJobService(sysTaskPlan.getTaskSoftTypeRdd())
                    .editTask(sysTask.getTaskPackage(), sysTask.getTaskClass(), sysTaskPlan.getName(),
                            sysTaskPlan.getGroupName(), sysTaskCycle.getTaskCycleExps(), sysTaskPlan.getTaskPlanParam());
            if (!baseResultBean.isSuccess()) {
                resultBean = ResultBean.fail(baseResultBean.getErrorMsg());
            }

            // 是否立即执行
            if (isExecute != null && isExecute == CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()) {
                baseResultBean = getJobService(sysTaskPlan.getTaskSoftTypeRdd()).executeJob(sysTaskPlan.getName(),
                        sysTaskPlan.getGroupName());
                if (!baseResultBean.isSuccess()) {
                    resultBean = ResultBean.fail(baseResultBean.getErrorMsg());
                }
            }
            return resultBean;
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 删除定时任务呢
     *
     * @param id 定时任务id
     * @return 处理结果
     */
    @DeleteMapping(value = "/delete/{id}")
    @ApiOperation(value = "删除任务计划", notes = "删除任务计划")
    public ResultBean deleteSysTaskTime(@PathVariable("id") String id) {
        try {
            startMultiService();

            // 数据校验
            ValidatorBean.checkNotNull(id, "任务计划id 不能为空");
            SysTaskPlan sysTaskPlan = sysTaskPlanService.getSysTaskPlanById(Long.parseLong(id));
            ValidatorBean.checkNotNull(sysTaskPlan, "定时任务计划不存在");

            sysTaskPlanService.deleteSysTaskPlanById(Long.parseLong(id));
            getJobService(sysTaskPlan.getTaskSoftTypeRdd()).deleteTask(sysTaskPlan.getName(), sysTaskPlan.getGroupName());

            return ResultBean.success().setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询所有任务计划
     *
     * @return 处理结果
     */
    @GetMapping(value = "/list")
    @ApiOperation(value = "查询所有任务计划", notes = "查询所有任务计划")
    public ResultBean findSysTaskTimeAll() {
        try {
            List<SysTaskPlan> list = sysTaskPlanService.listSysTaskPlan();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(list);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询任务计划
     *
     * @param id 任务计划id
     * @return 处理结果
     */
    @GetMapping(value = "/get/{id}")
    @ApiOperation(value = "查询任务计划", notes = "查询根据ID查询任务计划")
    public ResultBean getBySysTaskTimeId(@PathVariable("id") String id) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "任务计划id 不能为空");

            SysTaskPlan sysTaskPlan = sysTaskPlanService.getSysTaskPlanById(Long.parseLong(id));
            ValidatorBean.checkNotNull(sysTaskPlan, "定时任务计划不存在");

            return ResultBean.success("操作成功").setResultObject(sysTaskPlan).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 修改任务计划状态
     *
     * @param idStr  任务计划id
     * @param status 任务计划状态
     * @return 处理结果
     */
    @PutMapping(value = "/status/{id}/{status}")
    @ApiOperation(value = "修改任务计划状态", notes = "根据ID修改任务计划状态")
    public ResultBean updateQuartzStatus(@PathVariable("id") String idStr, @PathVariable("status") Integer status) {
        try {
            startMultiService();

            // 数据校验
            ValidatorBean.checkNotNull(idStr, "id 不能为空");
            ValidatorBean.checkNotZero(status, "状态不能为空");

            SysTaskPlan sysTaskPlan = sysTaskPlanService.getSysTaskPlanById(Long.parseLong(idStr));
            ValidatorBean.checkNotNull(sysTaskPlan, "定时任务计划不存在");

            sysTaskPlanService.updateSysTaskPlanStatus(Long.parseLong(idStr), status, AuthUtil.getSessionUser());

            // 停止或启用对应模块中的定时任务
            if (CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue() == status.intValue()) {
                getJobService(sysTaskPlan.getTaskSoftTypeRdd()).resumeJob(sysTaskPlan.getName(), sysTaskPlan.getGroupName());
            } else if (CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue() == status.intValue()) {
                getJobService(sysTaskPlan.getTaskSoftTypeRdd()).pauseJob(sysTaskPlan.getName(), sysTaskPlan.getGroupName());
            }
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 任务计划复杂查询，分页，排序
     *
     * @param taskTime 查询条件
     * @param pager    分页信息
     * @return 处理结果
     */
    @GetMapping("/query")
    @ApiOperation(value = "任务计划复杂查询，分页，排序", notes = "任务计划复杂查询，分页，排序")
    public ResultBean querySysTaskTimeByPager(SysTaskPlan taskTime, Pager pager) {
        try {
            ListPager taskTimeListPager = sysTaskPlanService.querySysTaskPlanByPager(taskTime, pager);
            return ResultBean.success("操作成功").setListPager(taskTimeListPager).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 立即执行
     *
     * @param id 任务计划id
     * @return 处理结果
     */
    @GetMapping(value = "/execute/{id}")
    @ApiOperation(value = "立即执行", notes = "立即执行")
    public ResultBean executeBySysTaskTimeId(@PathVariable("id") String id) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "任务计划id 不能为空");
            SysTaskPlan sysTaskPlan = sysTaskPlanService.getSysTaskPlanById(Long.parseLong(id));

            // 执行定时任务
            BaseResultBean baseResultBean = getJobService(sysTaskPlan.getTaskSoftTypeRdd())
                    .executeJob(sysTaskPlan.getName(), sysTaskPlan.getGroupName());
            if (baseResultBean.isSuccess()) {
                return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            } else {
                return ResultBean.fail("操作失败").setErrorMsg(baseResultBean.errorMsg);
            }
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
}
