package cn.estsh.i3plus.core.apiservice.mq;

import cn.estsh.i3plus.core.api.iservice.busi.ISysFileAttachService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysFileService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysMessageService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.JsonUtilTool;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysFile;
import cn.estsh.i3plus.pojo.platform.bean.SysFileAttach;
import cn.estsh.i3plus.pojo.platform.bean.SysMessage;
import cn.estsh.i3plus.pojo.platform.bean.SysRefUserMessage;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.rabbitmq.client.Channel;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * @Description : SWEB通知处理队列
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2018-11-15 22:15
 * @Modify:
 **/
@ConditionalOnExpression("'${impp.mq.queue.sweb.notice}' == 'true'")
@Component
public class MessageSWebNoticeQueueReceiver {
    private static final Logger LOGGER = LoggerFactory.getLogger(MessageSWebNoticeQueueReceiver.class);

    @Autowired
    private ISysMessageService sysMessageService;

    @Autowired
    private ISysFileAttachService sysFileAttachService;

    @Autowired
    private ISysFileService sysFileService;

    /**
     * SWEB通知处理队列
     *
     * @param data      系统消息信息
     * @param channel   通道
     * @param message   消息
     */
    @RabbitListener(queues = PlatformConstWords.SWEB_NOTICE_QUEUE)
    public void processImppMail(String data, Channel channel, Message message) {
        LOGGER.info("【MQ-{}】 数据接收成功：{}", PlatformConstWords.SWEB_NOTICE_QUEUE, data);
        // 添加消息
        SysMessage msg = JsonUtilTool.decode(data, SysMessage.class);
        try {
            msg = sysMessageService.insertSysMessage(msg);

            // 添加附件信息
            if(msg.getSysFileAttachList() != null){
                SysFile sysFile;
                for (SysFileAttach sysFileAttach : msg.getSysFileAttachList()) {
                    sysFile = sysFileService.getSysFileById(sysFileAttach.getFileId());
                    if(sysFile != null){
                        sysFileAttach.setRefId(msg.getId());
                        sysFileAttach.setFileNameRdd(sysFile.getFileOriginalName());
                        sysFileAttach.setFilePathRdd(sysFile.getFilePath());
                        // 初始数据
                        ConvertBean.serviceModelInitialize(sysFileAttach, msg.getCreateUser());

                        sysFileAttachService.insertSysFileAttach(sysFileAttach);
                    }
                }
            }

            // 添加消息明细信息
            if (StringUtils.isNotBlank(msg.getMessageReceiversId())) {
                // 获取供应商信息 由string转换为json对象
                JSONArray userJsonArray = new JSONArray();
                try {
                    userJsonArray = JSON.parseArray(msg.getMessageReceiversId());
                }catch (JSONException e){
                    LOGGER.error(" JSON转换错误【{0}】：{1}", msg.getMessageReceiversId(), e.getMessage());
                }
                JSONObject userJsonObject;

                List<SysRefUserMessage> insertList = new ArrayList<>(userJsonArray.size());
                List<String> receiversId = new ArrayList();
                List<String> receiversName = new ArrayList();
                SysRefUserMessage refUserMessage;

                for (Object user : userJsonArray) {
                    // 约定格式：[{'id':'111','name':'张三'},{'id':'121','name':'张三2'}]
                    userJsonObject = ( JSONObject )user;
                    receiversId.add(userJsonObject.get("id").toString());
                    receiversName.add(userJsonObject.get("name").toString());

                    refUserMessage = new SysRefUserMessage();
                    refUserMessage.setMessageId(msg.getId());
                    refUserMessage.setMessageSoftType(msg.getMessageSoftType());
                    refUserMessage.setMessageTitleRdd(msg.getMessageTitle());
                    refUserMessage.setMessageTypeRdd(msg.getMessageType());
                    refUserMessage.setMessageSenderNameRdd(msg.getMessageSenderNameRdd());
                    refUserMessage.setReceiverId(Long.parseLong(userJsonObject.get("id").toString()));
                    refUserMessage.setReceiverNameRdd(userJsonObject.get("name").toString());
                    refUserMessage.setMessageStatus(ImppEnumUtil.MESSAGE_STATUS.UNREAD.getValue());
                    refUserMessage.setReceiverTime(TimeTool.getNowTime(true));
                    refUserMessage.setIsUrgent(msg.getIsUrgent());
                    // 初始数据
                    ConvertBean.serviceModelInitialize(refUserMessage, msg.getCreateUser());

                    insertList.add(refUserMessage);
                }

                sysMessageService.insertSysRefUserMessage(insertList);

                // 设置收件人名称及id
                msg.setMessageReceiversId(StringUtils.join(receiversId,","));
                msg.setMessageReceiversNameRdd(StringUtils.join(receiversName,","));
                sysMessageService.updateSysMessage(msg);
            }

            // 消息处理完成
            LOGGER.info("【MQ-{}】站内信{} DeliveryTag:{} 处理成功", PlatformConstWords.IMPP_MESSAGE_LETTER_QUEUE,
                    msg, message.getMessageProperties().getDeliveryTag());
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
        } catch (Exception e) {
            e.printStackTrace();
            try {
                if(msg.getId() != null) {
                    sysMessageService.deleteSysMessageById(msg.getId());
                }
                // 未成功处理，重新发送
                channel.basicNack(message.getMessageProperties().getDeliveryTag(), false, true);
            } catch (IOException e1) {
                e1.printStackTrace();
            }
        }
    }

}
